#version 120

#define DYNAMIC_AMBIENT

#define LAVA_BRIGHTNESS 2.0
#define LAVA_FOG_DENSITY 0.3

// Increased torch values for better night visibility
#define TORCH_POWER 2.0
#define TORCH_INTENSITY 17.0
#define MIN_NIGHT_BRIGHTNESS 0.25

const int RGBA8 = 1;
const int RGB16 = 2;
const int RGBA16 = 3;
const int gcolorFormat = RGBA16;
const int gdepthFormat = RGBA8;
const int gnormalFormat = RGB16;

const vec3 torchcolor = vec3(1.2, 0.39, 0.02);

varying vec4 texcoord;
varying vec4 heldLight;

varying vec3 lightVector;
varying vec3 lightOffset0;
varying vec3 lightOffset3;
varying vec3 sunlight_color;
varying vec3 ambient_color;

varying float transition_fading;
varying float ambient_transition;
varying float gp00;
varying float gp11;
varying float gp22;
varying float gp32;
varying float gp2232;
varying vec2 igp;
varying float flicker;

uniform sampler2D gcolor;
uniform sampler2D gdepth;
uniform sampler2D gnormal;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D depthtex0;
uniform sampler2D depthtex2;
uniform sampler2D watershadow;
uniform sampler2D noisetex;

uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;

uniform vec3 cameraPosition;

uniform float far;
uniform float near;
uniform float rainStrength;
uniform float frameTimeCounter;
uniform float viewWidth;
uniform float viewHeight;

uniform int worldTime;

float calculateDayFactor() {
    float time = float(worldTime);
    if(time >= 23000.0 || time < 12000.0) {
        if(time >= 23000.0) {
            return (time - 23000.0) / 1000.0;
        } else {
            return min(1.0, time < 6000.0 ? time / 6000.0 : (12000.0 - time) / 6000.0);
        }
    }
    return 0.0;
}

float calculateNightFactor() {
    float time = float(worldTime);
    if(time >= 13000.0 && time < 23000.0) {
        float midnight = 18000.0;
        float nightStrength = time < midnight ? 
            min(1.0, (time - 13000.0) / 5000.0) : 
            min(1.0, (23000.0 - time) / 5000.0);
        return max(MIN_NIGHT_BRIGHTNESS, nightStrength);
    }
    return 0.0;
}

float calculateTransitionFactor() {
    float time = float(worldTime);
    if(time >= 12000.0 && time < 13000.0) {
        return (time - 12000.0) / 1000.0;
    } else if(time >= 23000.0) {
        return 1.0 - (time - 23000.0) / 1000.0;
    }
    return 0.0;
}

vec3 proj(in vec3 pos) {
    return vec3(pos.x * gp00, pos.y * gp11, gp2232) / pos.z + 0.5;
}

vec3 invproj(in vec3 p) {
    vec3 pos = p - 0.5;
    float z = gp32 / (pos.z + gp22);
    return vec3(pos.xy * igp, -1.0) * z;
}

float ld(in float depth) {
    return (2.0 * near) / (far + near - (2.0 * depth - 1.0) * (far - near));
}

float noise(in vec3 x) {
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f * f * (3.0 - 2.0 * f);
    vec2 uv = (p.xy + vec2(37.0, 17.0) * p.z) + f.xy;
    return mix(texture2D(noisetex, uv).x, 
               texture2D(noisetex, (uv + vec2(37.0, 17.0))).x, f.z);
}

void main() {
    vec3 color = texture2D(gcolor, texcoord.st).rgb;
    float spec = 0.0;

    vec4 aux = texture2D(gaux1, texcoord.st);
    
    vec3 eye = invproj(vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r));
    vec3 neye = normalize(eye);
    vec4 wpos = gbufferModelViewInverse * vec4(eye, 1.0);
    float dist = length(eye);
    vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;
    vec3 nnormal = normalize(normal);
    
    float no_hand = float(aux.g < 0.35 || 0.45 < aux.g);

    float dayFactor = calculateDayFactor();
    float nightFactor = calculateNightFactor();
    float transitionFactor = calculateTransitionFactor();

    float fogStart = 0.2;
    float fogEnd = 0.8;
    float fogDensity = mix(0.0, 0.008, rainStrength);

    vec3 dayFogColor = vec3(0.8863, 0.8863, 0.9412);
    vec3 nightFogColor = vec3(0.04, 0.04, 0.08); // Lighter night fog
    vec3 sunsetFogColor = vec3(0.8392, 0.6784, 0.6157);
    vec3 rainFogColor = vec3(0.8, 0.85, 0.9);

    vec3 finalFogColor;
    if (transitionFactor > 0.0) {
        finalFogColor = mix(
            mix(dayFogColor, nightFogColor, nightFactor),
            sunsetFogColor,
            transitionFactor
        );
    } else {
        finalFogColor = mix(nightFogColor, dayFogColor, dayFactor);
    }

    finalFogColor = mix(finalFogColor, rainFogColor, rainStrength * dayFactor);
    finalFogColor = mix(finalFogColor, nightFogColor, rainStrength * nightFactor * 1.2);

    float fogFactor = clamp((dist - fogStart) / (fogEnd - fogStart), 0.0, 1.0);
    fogFactor = 1.0 - exp(-fogDensity * dist);
    color = mix(color, finalFogColor, fogFactor);

    vec3 dayColor = vec3(1.15, 1.15, 1.1); // Increased day brightness
    vec3 nightColor = vec3(0.25, 0.25, 0.3); // Increased night brightness
    vec3 rainDayColor = vec3(0.8, 0.85, 0.9);
    
    vec3 timeAdjustment = mix(nightColor, dayColor, dayFactor);
    timeAdjustment = mix(timeAdjustment, rainDayColor, rainStrength * dayFactor);
    timeAdjustment = mix(timeAdjustment, vec3(0.15, 0.15, 0.2), rainStrength * nightFactor * 1.2);
    
    color *= timeAdjustment;

    float height = wpos.y;
    vec3 heightAdjustment = mix(
        vec3(1.0),
        vec3(1.05, 1.02, 0.98),
        clamp((height - 64.0) / 64.0, 0.0, 1.0)
    );
    color *= heightAdjustment;

    if (0.15 < aux.g) {
        float ao = aux.r;
        color *= mix(1.0, 0.8 + 0.2 * ao, 0.8);

        vec3 worldPos = wpos.xyz + cameraPosition;
        float terrainNoise = noise(worldPos * 0.5) * 0.05;
        color *= (1.0 + terrainNoise);
    }

    vec3 heldLightColor = heldLight.rgb * mix(TORCH_INTENSITY, TORCH_INTENSITY * 1.8, nightFactor);
    float heldLightFalloff = 1.0 / (1.0 + dist * dist * 0.01);
    color += heldLightColor * heldLightFalloff * heldLight.a;

    color = clamp(color, 0.0, 1.0);

    vec2 screenPos = gl_FragCoord.xy / vec2(viewWidth, viewHeight);
    float vignette = 1.0 - length(screenPos - 0.5) * 0.5;
    color *= mix(1.0, vignette, 0.3);

    gl_FragData[0] = vec4(color, spec);
}
