#version 120

#define gbuffers_textured
#include "shaders.settings"

// Varying inputs
varying vec4 color;
varying vec3 vworldpos;
varying mat3 tbnMatrix;
varying vec2 texcoord;
varying vec2 lmcoord;
varying float iswater;
varying float mat;
varying float NdotLight;

// Uniforms
uniform sampler2D normals;
uniform sampler2D texture;
uniform sampler2D lightmap;
uniform float rainStrength;
uniform vec4 entityColor;
uniform int isEyeInWater;
uniform int entityId;
uniform ivec2 eyeBrightnessSmooth;
uniform vec3 shadowLightPosition;
uniform float frameTimeCounter;
uniform float viewWidth;
uniform float viewHeight;

// Fog constants
#ifdef Fog
const int GL_LINEAR = 9729;
const int GL_EXP = 2048;
uniform int fogMode;
#endif

// Normal mapping function
#if nMap == 1
vec4 applyNormalMapping(vec4 tex) {
    vec3 normalMap = normalize((texture2D(normals, texcoord.xy).xyz * 2.0 - 1.0) * tbnMatrix);
    float lightIntensity = max(dot(normalMap, normalize(shadowLightPosition)), 0.0);
    tex.rgb *= (1.25 + lightIntensity * 0.15);
    
    #ifdef draw_bmap
    tex.rgb = normalMap;
    #endif
    
    return tex;
}
#endif

// Normal encoding for deferred rendering
vec4 encodeNormal(vec3 n) {
    return vec4(n.xy * inversesqrt(n.z * 8.0 + 8.0) + 0.5, mat/2.0, 1.0);
}

// Custom lighting calculation
#ifdef customLight
vec3 calculateCustomLighting(float torchLight, float skyLight) {
    vec3 emissiveColor = vec3(emissive_R, emissive_G, emissive_B);
    return mix(
        texture2D(lightmap, vec2(0.5 / 16.0, skyLight)).rgb,
        emissiveColor * torchLight * 1.2,
        torchLight
    );
}
#endif

// Water effect processing
vec4 processWaterEffect(vec4 tex) {
    if (isEyeInWater > 0.9) {
        tex.a = 0.9;
    }
    
    #ifdef waterTex
    tex.rgb *= 1.35;
    #else
        #if MC_VERSION < 11300
        tex = mix(tex, vec4(0.0, 0.3, 0.55, 0.75), 1.0) * texture2D(lightmap, lmcoord.st);
        #else
        tex.rgb = mix(tex.rgb, color.rgb * 0.6, 1.0) * texture2D(lightmap, lmcoord.st).rgb;
        #endif
    #endif
    
    return tex;
}

// Fog application
#ifdef Fog
vec3 applyFogEffect(vec3 color) {
    vec3 foggedColor = color;
    float fogFactor = 0.0;
    
    if (fogMode == GL_EXP || isEyeInWater == 1.0 || isEyeInWater == 2.0) {
        fogFactor = 1.0 - clamp(exp(-gl_Fog.density * gl_FogFragCoord), 0.0, 1.0);
    } else if (fogMode == GL_LINEAR) {
        fogFactor = clamp((gl_FogFragCoord - gl_Fog.start) * gl_Fog.scale, 0.0, 1.0);
    }
    
    return mix(color, gl_Fog.color.rgb, fogFactor);
}
#endif

void main() {
    // Basic texture sampling
    #ifndef customLight
    vec4 tex = texture2D(texture, texcoord.st) * texture2D(lightmap, lmcoord.xy) * color;
    #else
    vec4 tex = texture2D(texture, texcoord.st) * color;
    float torchLight = clamp(lmcoord.x - 0.5/16.0, 0.0, 1.0);
    tex.rgb *= calculateCustomLighting(torchLight, lmcoord.y);
    #endif

    // Initialize normal data
    vec4 normal = vec4(0.0);

    // Apply normal mapping if enabled
    #if nMap == 1
    tex = applyNormalMapping(tex);
    #endif

    // Apply directional lighting
    tex.rgb *= (1.0 + NdotLight) * 0.7;

    // Color enhancement
    #ifdef Colorboost
    tex.rgb = pow(tex.rgb * 1.25, vec3(1.15));
    #endif

    // Entity color mixing for mob damage flash
    #ifdef MobsFlashRed
    tex.rgb = mix(tex.rgb, entityColor.rgb, entityColor.a);
    #endif

    // Water processing
    if (iswater > 0.1) {
        tex = processWaterEffect(tex);
    }

    // Special case for entity ID 11000
    if (entityId == 11000.0) {
        tex = texture2D(lightmap, lmcoord.xy) * color;
    }

    // Make rain white - check for rain particle characteristics
    vec4 texColor = texture2D(texture, texcoord.st);
    if (rainStrength > 0.0 && texColor.a < 0.95 && texColor.a > 0.05) {
        float brightness = dot(tex.rgb, vec3(0.2126, 0.7152, 0.0722));
        if (brightness > 0.1 && brightness < 0.9) {
            tex.rgb = vec3(0.6275, 0.7137, 1.0);  // White color
            tex.a *= 0.7;  // Adjust transparency
        }
    }

    // Output data
    gl_FragData[0] = tex;
    gl_FragData[1] = encodeNormal(normal.xyz);

    // Apply fog if enabled
    #ifdef Fog
    gl_FragData[0].rgb = applyFogEffect(gl_FragData[0].rgb);
    #endif
}