#version 120

#define gbuffers_textured
#include "shaders.settings"

// Entity type definitions
const float ENTITY_IDS = float[](
    10031.0, // SMALLGRASS
    10175.0, // LOWERGRASS
    10176.0, // UPPERGRASS
    10059.0, // SMALLENTS
    10018.0, // LEAVES
    10106.0, // VINES
    10008.0, // WATER
    10111.0, // LILYPAD
    10079.0, // ICE
    10051.0, // FIRE
    10010.0, // LAVA
    10089.0, // EMISSIVE
    10091.0, // SOULFIRE
    10090.0, // WAVING_LANTERN
    10177.0  // INVERTED_LOWER
)[0]; // Using [0] to avoid GLSL array initialization issues

// Varying outputs
varying vec4 color;
varying vec3 vworldpos;
varying mat3 tbnMatrix;
varying vec2 texcoord;
varying vec2 lmcoord;
varying float iswater;
varying float mat;
varying float NdotLight;
varying vec4 screenPos; // Added for godrays

// Attributes
attribute vec4 mc_Entity;
attribute vec4 mc_midTexCoord;
attribute vec4 at_tangent;

// Uniforms
uniform vec3 cameraPosition;
uniform vec3 shadowLightPosition;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform float frameTimeCounter;

#ifdef TAA
uniform float viewWidth;
uniform float viewHeight;
uniform int framemod8;
vec2 texelSize = vec2(1.0/viewWidth, 1.0/viewHeight);
const vec2[8] offsets = vec2[8](
    vec2(1./8.,-3./8.),
    vec2(-1.,3.)/8.,
    vec2(5.0,1.)/8.,
    vec2(-3,-5.)/8.,
    vec2(-5.,5.)/8.,
    vec2(-7.,-1.)/8.,
    vec2(3,7.)/8.,
    vec2(7.,-7.)/8.
);
#endif

#ifdef HandLight
uniform int isEyeInWater;
uniform int heldBlockLightValue;
uniform int heldBlockLightValue2;
#endif

// Animation constants
const float PI = 3.14159265359;
float animTime = 150.79 * frameTimeCounter * animationSpeed;

// Wave calculation functions
vec3 calculateWaveOffset(vec3 pos, float freq, float maxMag, float minMag, float f0, float f1, float f2, float f3, float f4, float f5) {
    float magnitude = sin(animTime * freq + dot(pos, vec3(0.5))) * maxMag + minMag;
    vec3 phase = sin(vec3(f0, f1, f2) * animTime);
    
    vec3 offset;
    offset.x = animTime * f3 + phase.x + phase.y - pos.x + pos.z + pos.y;
    offset.z = animTime * f4 + phase.y + phase.z + pos.x - pos.z + pos.y;
    offset.y = animTime * f5 + phase.z + phase.x + pos.z + pos.y - pos.y;
    
    return sin(offset) * magnitude;
}

vec3 calculateMovement(vec3 pos, float f0, float f1, float f2, float f3, float f4, float f5, vec3 amp1, vec3 amp2) {
    vec3 move1 = calculateWaveOffset(pos, 0.0027, 0.0500, 0.0500, 0.0127, 0.0089, 0.0114, 0.0063, 0.0224, 0.0015) * amp1;
    vec3 move2 = calculateWaveOffset(pos + move1, 0.0348, 0.0500, 0.0500, f0, f1, f2, f3, f4, f5) * amp2;
    return move1 + move2;
}

void applyWaving(inout vec3 position, vec3 worldPos, bool isTop) {
    if (isTop) {
        #ifdef Waving_Grass
        if (mc_Entity.x == 10031.0) // SMALLGRASS
            position.xyz += calculateMovement(worldPos, 0.0041, 0.0070, 0.0044, 0.0038, 0.0063, 0.0000, vec3(3.2,1.8,3.2), vec3(0.0));
        #endif

        #ifdef Waving_Entities
        if (mc_Entity.x == 10059.0) // SMALLENTS
            position.xyz += calculateMovement(worldPos, 0.0041, 0.0070, 0.0044, 0.0038, 0.0240, 0.0000, vec3(1.0,0.0,1.0), vec3(0.5,0.0,0.5));
        #endif

        #ifdef Waving_Fire
        if (mc_Entity.x == 10051.0) // FIRE
            position.xyz += calculateMovement(worldPos, 0.0105, 0.0096, 0.0087, 0.0063, 0.0097, 0.0156, vec3(1.4,0.4,1.4), vec3(0.8));
        #endif
    }

    #ifdef Waving_Tallgrass
    if ((mc_Entity.x == 10175.0 && isTop) || mc_Entity.x == 10176.0) // LOWERGRASS & UPPERGRASS
        position.xyz += calculateMovement(worldPos, 0.0041, 0.0070, 0.0044, 0.0038, 0.0240, 0.0000, vec3(1.0,0.0,1.0), vec3(0.5,0.0,0.5));
    #endif

    #ifdef Waving_Leaves
    if (mc_Entity.x == 10018.0) // LEAVES
        position.xyz += calculateMovement(worldPos, 0.0040, 0.0064, 0.0043, 0.0035, 0.0037, 0.0041, vec3(1.2,0.3,1.2), vec3(0.6,0.2,0.6));
    #endif

    #ifdef Waving_Vines
    if (mc_Entity.x == 10106.0) // VINES
        position.xyz += calculateMovement(worldPos, 0.0040, 0.0064, 0.0043, 0.0035, 0.0037, 0.0041, vec3(0.6,1.2,0.6), vec3(0.3,0.6,0.3));
    #endif
}

void main() {
    // Basic texture coordinates setup
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    
    #ifdef HandLight
    if(!(isEyeInWater == 1 || isEyeInWater == 2)) {
        lmcoord.x = max(lmcoord.x, max(float(heldBlockLightValue), float(heldBlockLightValue2)) - 1.0 - length(gl_ModelViewMatrix * gl_Vertex) / 15.0);
    }
    #endif
    
    // Position calculations
    vec3 position = mat3(gbufferModelViewInverse) * (gl_ModelViewMatrix * gl_Vertex).xyz + gbufferModelViewInverse[3].xyz;
    vworldpos = position.xyz + cameraPosition;
    bool isTop = gl_MultiTexCoord0.t < mc_midTexCoord.t;

    // Apply waving animations
    applyWaving(position, vworldpos, isTop);

    // Water and lava effects
    iswater = (mc_Entity.x == 10008.0) ? 0.95 : 0.0;

    #ifdef Waving_Water
    if(mc_Entity.x == 10008.0 || mc_Entity.x == 10111.0) {
        float wave = 0.05 * sin(2.0 * PI * (frameTimeCounter * 0.8 + vworldpos.x / 2.5 + vworldpos.z / 5.0))
                  + 0.05 * sin(2.0 * PI * (frameTimeCounter * 0.6 + vworldpos.x / 6.0 + vworldpos.z / 12.0));
        position.y += clamp(wave, -fract(vworldpos.y + 0.001), 1.0-fract(vworldpos.y + 0.001)) * waves_amplitude * 1.2;
    }
    #endif

    // Final position and TAA
    gl_Position = gl_ProjectionMatrix * gbufferModelView * vec4(position, 1.0);
    screenPos = gl_Position; // Store screen position for godrays
    #ifdef TAA
    gl_Position.xy += offsets[framemod8] * gl_Position.w * texelSize;
    #endif

    // Fog and color
    gl_FogFragCoord = length(position.xyz);
    color = gl_Color;

    // Fix emissive block colors
    if(mc_Entity.x == 10089.0 || mc_Entity.x == 10091.0 || mc_Entity.x == 10010.0 || 
       mc_Entity.x == 10051.0 || mc_Entity.x == 10090.0 || mc_Entity.x == 10300.0) {
        color = vec4(1.0);
    }
    if(mc_Entity.x == 10091.0 || mc_Entity.x == 10090.0) lmcoord.x = 1.0;

    // Normal mapping setup
    vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
    vec3 tangent = normalize(gl_NormalMatrix * at_tangent.xyz);
    vec3 binormal = normalize(gl_NormalMatrix * cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
    tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
                     tangent.y, binormal.y, normal.y,
                     tangent.z, binormal.z, normal.z);

    // Light calculations
    NdotLight = clamp(dot(normal, normalize(shadowLightPosition)) * 1.05 - 0.05, 0.0, 1.0);
    
    // Override lighting for specific entities
    if (mc_Entity.x == 10031.0 || mc_Entity.x == 10175.0 || mc_Entity.x == 10176.0 || 
        mc_Entity.x == 10059.0 || mc_Entity.x == 10018.0 || mc_Entity.x == 10106.0 || 
        mc_Entity.x == 10111.0 || mc_Entity.x == 10051.0 || mc_Entity.x == 10090.0 || 
        mc_Entity.x == 10089.0 || mc_Entity.x == 10030.0 || mc_Entity.x == 10115.0 || 
        mc_Entity.x == 10576.0 || mc_Entity.x == 10006.0) {
        NdotLight = 0.65;
    }
}